@@ -190,6 +190,11 @@ void RegressionSplittingRule::find_best_split_value_large_q(const Data& data,
 
   // Compute decrease of impurity for each split
   for (size_t i = 0; i < num_unique - 1; ++i) {
+    // Continue if nothing here
+    if (counter[i] == 0) {
+      continue;
+    }
+    
     n_left += counter[i];
     sum_left += sums[i];
 
